
* {box-sizing: border-box}

/* Container needed to position the overlay. Adjust the width as needed */
.showcase-container {
  position: relative;
  width: 100%;
  max-width: 300px;
}

.showcase-container .content {
  position: absolute; /* Position the background text */
  bottom: 0; /* At the bottom. Use top:0 to append it to the top */
  background: rgb(0, 0, 0); /* Fallback color */
  background: rgba(0, 0, 0, 0.5); /* Black background with 0.5 opacity */
  color: #f1f1f1; /* Grey text */
  width: 100%; /* Full width */
  padding: 2px; /* Some padding */
}

/* Make the image to responsive */
.image {
  display: block;
  width: 100%;
  height: auto;
}

/* The overlay effect - lays on top of the container and over the image */
.overlay {
  position: absolute;
  bottom: 0;
  background: #313131;
  background: rgba(45, 45, 45, 0.98); /* Black see-through */
  color: #a96060;
  width: 100%;
  transition: .5s ease;
  opacity:0;
  color: white;
  font-size: 20px;
  padding: 20px;
  text-align: center;
}




/* When you mouse over the container, fade in the overlay title */
.container:hover .overlay {
  opacity: 1;
}

/* Three image containers (use 25% for four, and 50% for two, etc) */
.column {
  float: left;
  width: 33%;
  padding: 5px;
}

/* Centered text */
.centered {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
}

.column1 {
  float: left;
  width: 20%;
  padding: 5px;
}

/* Clear floats after image containers */
.row::after {
  content: "";
  clear: both;
  display: table;
}
.card {
  box-shadow: 0 4px 8px 0 rgba(0, 0, 0, 0.2);
  max-width: 300px;
  margin: auto;
  text-align: center;
  font-family: arial;
}

.card button {
  border: none;
  outline: 0;
  padding: 12px;
  color: white;
  background-color: #313131;
  text-align: center;
  cursor: pointer;
  width: 100%;
  font-size: 18px;
  transition-duration: color 125ms;
  cursor: pointer
}

.card button:hover {
  background-color: #414141;
  color: #f7fafd;
}
/* Solid border */
hr.solid {
  border-top: 1px solid #2d2d2d;
}
