### {{ param.opt_str }}

{% if param.required %}
"**REQUIRED**"
{% endif %}

{% if param.envvar %}
"**REQUIRED**"
{% endif %}

{% if param.multiple %}
**Multiple values allowed.**
{% endif %}

{% if param.default %}
**Default:**: {{ param.default }}
{% endif %}

{% if param.is_flag %}
**Flag:**: {{ param.help }}
{% endif %}

{% if param.help %}
{{ param.help }}
{% endif %}
