@keyframes heart {
    0%, 40%, 80%, to {
    transform: scale(1);
}
20%, 60% {
    transform: scale(1.15);
}
}.md-typeset .twitter {
    color: #00acee;
}
.md-typeset .mastodon {
    color: #897ff8;
}
.md-typeset .mdx-video {
    width: auto;
}
.md-typeset .mdx-video__inner {
    height: 0;
    padding-bottom: 56.138%;
    position: relative;
    width: 100%}
.md-typeset .mdx-video iframe {
    border: none;
    height: 100%;
    left: 0;
    overflow: hidden;
    position: absolute;
    top: 0;
    width: 100%}
.md-typeset .mdx-heart {
    animation: heart 1s infinite;
}


