##Description

Every [MkPage][mknodes.MkPage] as well as every [MkNav][mknodes.MkNav] can carry metadata.
Page metadata inherits from Nav metadata, similarly as the **MkDocs-Material** meta plugin.

Metadata can be set either via the [MkPage][mknodes.MkPage] constructor, via the [MkNav.route.page][mknodes.navs.navrouter.NavRouter.page]
decorator. Also, some methods like [MkNav.parse.folder][mknodes.navs.navparser.NavParser.folder] take metadata keyword arguments
in order to set it for all parsed pages.

## Examples

Via decorators:
`````py
@nav.route.page("My page", icon="material/code-json", status="new")
def _(page: mk.MkPage):
    ...
`````

Via constructor:
`````py
page = MkPage("My page", icon="material/code-json", status="new")
`````

## Metadata fields

{{ "mknodes.Metadata" | MkDocStrings(show_root_toc_entry=False, heading_level=3, show_bases=False, show_source=False) }}
