{% for admonition in admonitions %}
.admonition.{{ admonition.name }}, details.{{ admonition.name }} {
    color: {{ admonition.font_color}};
    background-color: {{ admonition.header_color }};
    border-color: {{ admonition.border_color }};
}
{% endfor %}

{% if alternating_table_colors %}
.table tr:nth-child(2n) {
    background-color: #CCCCCCAA;
}
{% endif %}
