MkNavs can parse index files in `SUMMARY.md` style (as popularized by **mkdocs-literate-nav**).

Given the following folder structure:

{{ path | MkTreeView | MkAdmonition(title="Directory tree") }}

and a root `SUMMARY.md` looking like:

{{ text | MkCode | MkAdmonition(title="Root SUMMARY.md") }}

